#ifndef OLD

#include <Arduino.h>

#define PIN_ANALOG_CURRENT  0
#define PIN_ON_OFF          2
#define PIN_JACK            3
#define PIN_RELAY_PUMP1     4
#define PIN_BT_START        5
#define PIN_RF              6
#define PIN_RELAY_PUMP2     7
#define PIN_RELAY_POWER     8
#define PIN_LED             9
#define PIN_LED_ON          11
#define PIN_LED_START       10

#define ALPHA               0.900f


// states
enum state {
    STATE_ON, START_JET, STOP_JET, STATE_ASPI, STATE_OFF, TURN_OFF
};

// define state buttons
uint32_t now;
uint32_t led_now;
float prev_current = 0.0f;
int pinState = 0;

state nState = STATE_ON;
bool on_off_released = false;


bool checkInput(){
    return ((digitalRead(PIN_BT_START) == LOW) || (digitalRead(PIN_RF) == LOW) || (digitalRead(PIN_JACK) == LOW));
}

bool checkOnOff(){
    return ((digitalRead(PIN_ON_OFF) == HIGH));
}

void setup() {
    // initialize pin
    pinMode(PIN_RELAY_POWER, OUTPUT);
    digitalWrite(PIN_RELAY_POWER, HIGH);
    pinMode(PIN_LED_ON, OUTPUT);
    digitalWrite(PIN_LED_ON, HIGH);
    pinMode(PIN_LED_START, OUTPUT);
    digitalWrite(PIN_LED_START, HIGH);
    Serial.begin(9600);
    pinMode(PIN_RELAY_PUMP1, OUTPUT);
    digitalWrite(PIN_RELAY_PUMP1, LOW);
    pinMode(PIN_RELAY_PUMP2, OUTPUT);
    digitalWrite(PIN_RELAY_PUMP2, LOW);
    pinMode(PIN_BT_START, INPUT_PULLUP);
    pinMode(PIN_JACK, INPUT_PULLUP);
    pinMode(PIN_RF, INPUT_PULLUP);
    pinMode(LED_BUILTIN, OUTPUT);
    pinMode(PIN_ON_OFF, INPUT);
    digitalWrite(LED_BUILTIN, pinState);
    //analogReference(INTERNAL);
    now = millis();
    led_now = millis();
    Serial.println("STATE_ON");
}

// the loop function runs over and over again forever
void loop() {
    switch (nState) {
    case   STATE_ON:
        digitalWrite(PIN_RELAY_POWER, HIGH);
        if (checkInput()){
            Serial.println("START_JET");
            nState = START_JET;
            now = millis();
        }
        if((uint32_t) (millis() - now) > 300000){
            Serial.println("STATE_OFF");
            nState = STATE_OFF;
            now = millis();
        }
        if(on_off_released && checkOnOff() && ((uint32_t) (millis() - now) > 2000)){
            Serial.println("STATE_OFF");
            nState = STATE_OFF;
            now = millis();
        }
        break;
    case   START_JET:
        digitalWrite(PIN_RELAY_PUMP1, HIGH);
        //digitalWrite(PIN_RELAY_PUMP1, LOW);
        digitalWrite(PIN_RELAY_PUMP2, LOW);
        if((uint32_t) (millis() - now) > 60000){
            Serial.println("STOP_JET 60 000");
            nState = STOP_JET;
            now = millis();
        }
        {
           int current = analogRead(PIN_ANALOG_CURRENT);
           prev_current = ALPHA * prev_current + (1.0f - ALPHA) * current;
            Serial.println(prev_current);
            
            //Serial.print(",");
            if(((uint32_t) (millis() - now) > 5000) && (prev_current < 532)){
                Serial.println("STOP_JET532");
                nState = STOP_JET;
                now = millis();
            }
        }
        if(((uint32_t) (millis() - now) > 1000) && (checkInput() || checkOnOff())){
            Serial.println("STOP_JET 1000");
            nState = STOP_JET;
            now = millis();
        }
        break;
    case STOP_JET:
        digitalWrite(PIN_RELAY_PUMP1, LOW);
        digitalWrite(PIN_RELAY_PUMP2, LOW);
        if((uint32_t) (millis() - now) > 2000){
            Serial.println("STATE_ASPI");
            nState = STATE_ASPI;
            now = millis();
        }
        break;
    case   STATE_ASPI:
        digitalWrite(PIN_RELAY_PUMP2, HIGH);
        digitalWrite(PIN_RELAY_PUMP1, LOW);
        if((uint32_t) (millis() - now) > 5000){
            Serial.println("STATE_OFF");
            nState = STATE_OFF;
            now = millis();
        }
        break;
    case   STATE_OFF:
        digitalWrite(PIN_RELAY_PUMP2, LOW);
        digitalWrite(PIN_RELAY_PUMP1, LOW);
        if((uint32_t) (millis() - now) > 5000){
            Serial.println("TURN_OFF");
            nState = TURN_OFF;
            now = millis();
        }
        break;
    case TURN_OFF:
        digitalWrite(PIN_RELAY_POWER, LOW);
        break;
    default: 
        Serial.println( nState, DEC);
        break;
    }

    if((uint32_t)(millis() - led_now) > 500){
        led_now = millis();
        pinState ^= 0x01;
        digitalWrite(LED_BUILTIN, pinState);
        if(nState == STATE_ON ||  nState == STOP_JET){
            digitalWrite(PIN_LED_START, pinState);
        }
        else if(nState == STATE_ASPI || nState == START_JET){
            digitalWrite(PIN_LED_START, HIGH);
        } else {
            digitalWrite(PIN_LED_START, LOW);
        }
        if(nState == STATE_OFF){
            digitalWrite(PIN_LED_ON, pinState);
        }

    }
    on_off_released |= (!checkOnOff());

}


#else
//a 2022/10/22: use Arduino
//#include<stdio.h>
#include <Arduino.h>

// define pin
// input
const int pin_bt_on = 10;
const int pin_bt_start = 5;

// output
const int pin_relay_pump1 = 4;
const int pin_relay_pump2 = 7;
const int pin_relay_power = 8;
const int pin_led = 9;

// states
enum state {
    STATE_ON = 10, STATE_JET, STATE_ASPI, STATE_OFF
};

// define state buttons
bool bt_on = LOW;
bool bt_start = LOW;

state nState = STATE_ON;

void setup() {
    // initialize pin
    Serial.begin(9600);
    pinMode(pin_relay_power, OUTPUT);
    digitalWrite(pin_relay_power, HIGH);
    pinMode(pin_relay_pump1, OUTPUT);
    digitalWrite(pin_relay_pump1, LOW);
    pinMode(pin_relay_pump2, OUTPUT);
    digitalWrite(pin_relay_pump2, LOW);
    pinMode(pin_bt_start, INPUT);
}

// the loop function runs over and over again forever
void loop() {
    Serial.println("I am LOOP\n");
    Serial.println((int)(nState), DEC);
    switch (nState) {
    case   STATE_ON:
        Serial.println("on");
        digitalWrite(pin_relay_power, HIGH);
        if (digitalRead(pin_bt_start) == LOW)
            nState = STATE_JET;
        break;
    case   STATE_JET:
        digitalWrite(pin_relay_pump1, HIGH);
        digitalWrite(pin_relay_pump2, LOW);
        delay(6000);
        digitalWrite(pin_relay_pump1, LOW);
        digitalWrite(pin_relay_pump2, LOW);
        delay(2000);
        nState = STATE_ASPI;
        break;
    case   STATE_ASPI:
        digitalWrite(pin_relay_pump2, HIGH);
        digitalWrite(pin_relay_pump1, LOW);
        delay(5000);
        nState = STATE_OFF;
        break;
    case   STATE_OFF:
        digitalWrite(pin_relay_pump2, LOW);
        digitalWrite(pin_relay_pump1, LOW);
        delay(5000);
        digitalWrite(pin_relay_power, LOW);
        break;
    default: Serial.println("I am default\n");
    }

    /*digitalWrite(LED_BUILTIN, HIGH);
        //digitalWrite(pin_led, HIGH);  // indicate via LED
    delay(100);
    digitalWrite(LED_BUILTIN, LOW);
    delay(500);*/
}
#endif